==> tecfy_corev2_package-1.1.66.tar.gz: lib/views/entity/entity_form.pade.dart <==
part of tecfy_corev2_client_package;

class EntityFormPage extends StatefulWidget {
  final String entity;
  final dynamic data;
  final bool isPopup;
  final bool readOnly;
  final bool loadData;

  /// Create a form to Add/Update Entity and return the boolean [true] if success
  EntityFormPage(this.entity,
      {Key? key,
      this.data,
      this.isPopup = false,
      this.readOnly = false,
      this.loadData = true})
      : super(key: key);
  @override
  _EntityFormPageState createState() => _EntityFormPageState();
}

class _EntityFormPageState
    extends TecfyStateView<EntityFormPage, EntityFormController> {
  bool _isSaved = false;
  bool _isMobile = false;
  var _blancData = {};

  Widget _getMenu() {
    return PopupMenuButton<String>(
      //onSelected: _controller.choicePopMenuAction,
      onSelected: (cmd) async {
        if (cmd == 'history') {
          _controller.onOpenHistory();
        } else if (cmd == 'SaveNew') {
          if (await _controller.onSaveNew()) setState(() => _isSaved = true);
        }
      },
      color: Color.fromRGBO(73, 77, 89, 1),
      itemBuilder: (BuildContext context) => <PopupMenuEntry<String>>[
        PopupMenuItem<String>(
          value: 'history',
          textStyle: TextStyle(color: ThemeService.textOnMainColor),
          child: Row(
            children: [
              Expanded(child: TecfyText(trans('History'))),
              Icon(
                Icons.sticky_note_2_sharp,
                color: ThemeService.textOnMainColor,
                semanticLabel: trans('History'),
              ),
            ],
          ),
          enabled: true,
        ),
      ]..addAll(!_controller.canSave
          ? []
          : [
              PopupMenuItem<String>(
                value: 'SaveNew',
                textStyle: TextStyle(color: ThemeService.textOnMainColor),
                child: Row(
                  children: [
                    Expanded(child: TecfyText(trans('Save New'))),
                    Icon(
                      Icons.add_task,
                      color: ThemeService.textOnMainColor,
                      semanticLabel: trans('Save New'),
                    ),
                  ],
                ),
                enabled: true,
              )
            ]),
    );
  }

  @override
  void initState() {
    _controller = EntityFormController(
        this, widget.entity, widget.data, widget.readOnly, widget.loadData);
    if (widget.data == null) {
      if (mapEquals(
              ConfigService.getValueMap(
                  "savedDataForm${_controller.entity.name}"),
              {}) ==
          false) {
        ///TODO: Remove إستعادة البيانات السابقة ؟
        // Future.delayed(Duration.zero, () async {
        //   var getTheData = await AppNavigator.confirmMessage(
        //       context, "أستعادة البيانات السابقة");
        //   if (getTheData) {
        //     _controller.data = ConfigService.getValueMap(
        //         "savedDataForm${_controller.entity.name}");
        //     _blancData = ConfigService.getValueMap(
        //         "savedDataForm${_controller.entity.name}");
        //     _controller.setState();
        //   } else {
        //     _blancData = jsonDecode(jsonEncode(_controller.data,
        //         toEncodable: EntityService.customEncode));
        //   }
        // });
      }
      // Future.delayed(Duration(microseconds: 300), () {
      //   _blancData = jsonDecode(jsonEncode(_controller.data,
      //       toEncodable: EntityService.customEncode));
      // });
    }
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    if (widget.data == null) {
      if (_isSaved == false) {
        bool equalityMap(var mapx, var mapy) {
          for (var k in mapx.keys) {
            if (mapx[k] == null || mapx[k].toString().length == 0) continue;
            if (mapy[k] != mapx[k]) {
              return false;
            }
          }
          return true;
        }

        if (!equalityMap(_controller.data, _blancData)) {
          ConfigService.setValueMap("savedDataForm${_controller.entity.name}",
              _controller.data ?? {});
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    _isMobile = widget.isPopup || LayoutService.isMobile(context);
    return DefaultTabController(
      length: _controller.tabs.length,
      child: BasePage(
        context,
        _controller.entity.name,
        _controller.entity.dbName,
        isMobile: _isMobile,
        appBarActions: <Widget>[
          !_controller.canSave
              ? Container()
              : _isMobile
                  ? IconButton(
                      icon: Icon(Icons.save, color: ThemeService.mainColor),
                      //color: ThemeService.secondryColor,
                      tooltip: trans('Save'),
                      onPressed: () async {
                        if (await _controller.onSave())
                          setState(() => _isSaved = true);
                      })
                  : AppButtonIconText(
                      icon:
                          Icon(Icons.save, color: ThemeService.textOnMainColor),
                      color: ThemeService.secondryColor,
                      label: trans('Save'),
                      onPressed: () async {
                        if (await _controller.onSave())
                          setState(() => _isSaved = true);
                      }),
          _getMenu()
        ],
        appBarBottom: _controller.tabs.length > 1
            ? _isMobile
                ? _getTabBar()
                : Align(
                    alignment: LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                    child: Container(
                        child: _getTabBar(),
                        constraints: BoxConstraints(
                            maxWidth:
                                (_controller.tabs.length * 150).toDouble())))
            : null,
        child: Align(
            alignment: Alignment.topCenter,
            child: Container(
              constraints:
                  BoxConstraints(maxWidth: ThemeService.containerWidth),
              child: _controller.tabs.length > 1
                  ? TabBarView(
                      children: _controller.tabs
                          .map<Widget>((tab) => _getTabData(tab))
                          .toList())
                  : _controller.tabs.length <= 0
                      ? SizedBox()
                      : _getTabData(_controller.tabs.first),
            )),
      ),
    );
  }

  TabBar _getTabBar() {
    return TabBar(
      automaticIndicatorColorAdjustment: true,
      indicatorSize: TabBarIndicatorSize.tab,
      isScrollable: _controller.tabs.length > 3 ? true : false,
      labelColor: ThemeService.mainColor,
      tabs: _controller.tabs
          .map((tab) => Tab(
                child: FittedBox(
                    fit: BoxFit.scaleDown,
                    child: Text(
                        LocalConfigService.isArabic ? tab.nameAr : tab.nameEn)),
              ))
          .toList(),
    );
  }

  Widget _getTabData(ViewTab tab) {
    if (_controller.properties.length == 0) return SizedBox();
    var props = _controller.getProps(tab);
    if (props.length == 0) {
      return SizedBox();
    }
    return SingleChildScrollView(child: _getData(props));
    // return SingleChildScrollView(
    //     child: props.length > 1
    //         ? Column(
    //             mainAxisAlignment: MainAxisAlignment.start,
    //             crossAxisAlignment: CrossAxisAlignment.start,
    //             children: group.map<Widget>((t) => _getData(t)).toList())
    //         : _getData(group.first));
  }

  _getData(List<Property> props) {
    props.sort((a, b) => a.editOrder - b.editOrder);
    return props.length == 1 // || isMobile
        ? LayoutBuilder(
            builder: (lbContext, boxConstraints) => Container(
                width: _getWidth(props.first.size, boxConstraints.maxWidth),
                margin: EdgeInsets.symmetric(horizontal: 10),
                child: PorpertyComponent(
                  readOnly: _controller.readOnly,
                  property: props.first,
                  isAddNew: _controller.isAddNew,
                  entity: _controller.entity,
                  //filterValue: _controller.data[props.first.filterValueField],
                  data: _controller.data!,
                  onChange: (v) => _controller.onValueChanging(
                      v,
                      props.first
                          .dbName), //_controller.data[props.first.dbName] = v,
                )))
        : LayoutBuilder(
            builder: (lbContext, boxConstraints) => Wrap(
              alignment: WrapAlignment.start,
              children: props
                  .map<Widget>((p) => Container(
                      width: _getWidth(p.size, boxConstraints.maxWidth),
                      margin: EdgeInsets.symmetric(horizontal: 10),
                      child: PorpertyComponent(
                        readOnly: _controller.readOnly,
                        isAddNew: _controller.isAddNew,
                        property: p,
                        entity: _controller.entity,
                        // onChange: (v) => _controller.data[p.dbName] = v,
                        //filterValue: _controller.data[p.filterValueField],
                        data: _controller.data!,
                        onChange: (v) =>
                            _controller.onValueChanging(v, p.dbName),
                      )))
                  .toList(),
            ),
          );
  }

  double? _getWidth(PropertySizes size, double fullwidth) {
    if (size == PropertySizes.MaxSize) return fullwidth;
    if (size == PropertySizes.xSmall) return 100;
    if (size == PropertySizes.Small) return 150;
    if (size == PropertySizes.Large) return 400;
    if (size == PropertySizes.xLarge) return 500;

    if (fullwidth < 400) {
      return fullwidth; // means take full width
    } else if (fullwidth < 850) {
      return (fullwidth / 2) - (_isMobile ? 20 : 20);
    }
    return (fullwidth / 3) - 20;
  }
}

==> tecfy_corev2_package-1.1.78.tar.gz: lib/views/entity/entity_form.pade.dart <==
part of tecfy_corev2_client_package;

class EntityFormPage extends StatefulWidget {
  final String entity;
  final dynamic data;
  final bool isPopup;
  final bool readOnly;
  final bool loadData;

  /// Create a form to Add/Update Entity and return the boolean [true] if success
  EntityFormPage(this.entity,
      {Key? key,
      this.data,
      this.isPopup = false,
      this.readOnly = false,
      this.loadData = true})
      : super(key: key);
  @override
  _EntityFormPageState createState() => _EntityFormPageState();
}

class _EntityFormPageState
    extends TecfyStateView<EntityFormPage, EntityFormController> {
  bool _isSaved = false;
  bool _isMobile = false;
  var _blancData = {};

  Widget _getMenu() {
    return PopupMenuButton<String>(
      //onSelected: _controller.choicePopMenuAction,
      onSelected: (cmd) async {
        if (cmd == 'history') {
          _controller.onOpenHistory();
        } else if (cmd == 'SaveNew') {
          if (await _controller.onSaveNew()) setState(() => _isSaved = true);
        }
      },
      color: Color.fromRGBO(73, 77, 89, 1),
      itemBuilder: (BuildContext context) => <PopupMenuEntry<String>>[
        PopupMenuItem<String>(
          value: 'history',
          textStyle: TextStyle(color: ThemeService.textOnMainColor),
          child: Row(
            children: [
              Expanded(child: TecfyText(trans('History'))),
              Icon(
                Icons.sticky_note_2_sharp,
                color: ThemeService.textOnMainColor,
                semanticLabel: trans('History'),
              ),
            ],
          ),
          enabled: true,
        ),
      ]..addAll(!_controller.canSave
          ? []
          : [
              PopupMenuItem<String>(
                value: 'SaveNew',
                textStyle: TextStyle(color: ThemeService.textOnMainColor),
                child: Row(
                  children: [
                    Expanded(child: TecfyText(trans('Save New'))),
                    Icon(
                      Icons.add_task,
                      color: ThemeService.textOnMainColor,
                      semanticLabel: trans('Save New'),
                    ),
                  ],
                ),
                enabled: true,
              )
            ]),
    );
  }

  @override
  void initState() {
    _controller = EntityFormController(
        this, widget.entity, widget.data, widget.readOnly, widget.loadData);
    if (widget.data == null) {
      if (mapEquals(
              ConfigService.getValueMap(
                  "savedDataForm${_controller.entity.name}"),
              {}) ==
          false) {
        ///TODO: Remove إستعادة البيانات السابقة ؟
        // Future.delayed(Duration.zero, () async {
        //   var getTheData = await AppNavigator.confirmMessage(
        //       context, "أستعادة البيانات السابقة");
        //   if (getTheData) {
        //     _controller.data = ConfigService.getValueMap(
        //         "savedDataForm${_controller.entity.name}");
        //     _blancData = ConfigService.getValueMap(
        //         "savedDataForm${_controller.entity.name}");
        //     _controller.setState();
        //   } else {
        //     _blancData = jsonDecode(jsonEncode(_controller.data,
        //         toEncodable: EntityService.customEncode));
        //   }
        // });
      }
      // Future.delayed(Duration(microseconds: 300), () {
      //   _blancData = jsonDecode(jsonEncode(_controller.data,
      //       toEncodable: EntityService.customEncode));
      // });
    }
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    if (widget.data == null) {
      if (_isSaved == false) {
        bool equalityMap(var mapx, var mapy) {
          for (var k in mapx.keys) {
            if (mapx[k] == null || mapx[k].toString().length == 0) continue;
            if (mapy[k] != mapx[k]) {
              return false;
            }
          }
          return true;
        }

        if (!equalityMap(_controller.data, _blancData)) {
          ConfigService.setValueMap("savedDataForm${_controller.entity.name}",
              _controller.data ?? {});
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    _isMobile = widget.isPopup || LayoutService.isMobile(context);
    return DefaultTabController(
      length: _controller.tabs.length,
      child: BasePage(
        context,
        _controller.entity.name,
        _controller.entity.dbName,
        isMobile: _isMobile,
        appBarActions: <Widget>[
          !_controller.canSave
              ? Container()
              : _isMobile
                  ? IconButton(
                      icon: Icon(Icons.save, color: ThemeService.mainColor),
                      //color: ThemeService.secondryColor,
                      tooltip: trans('Save'),
                      onPressed: () async {
                        if (await _controller.onSave())
                          setState(() => _isSaved = true);
                      })
                  : AppButtonIconText(
                      icon:
                          Icon(Icons.save, color: ThemeService.textOnMainColor),
                      color: ThemeService.secondryColor,
                      label: trans('Save'),
                      onPressed: () async {
                        if (await _controller.onSave())
                          setState(() => _isSaved = true);
                      }),
          _getMenu()
        ],
        appBarBottom: _controller.tabs.length > 1
            ? _isMobile
                ? _getTabBar()
                : Align(
                    alignment: LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                    child: Container(
                        child: _getTabBar(),
                        constraints: BoxConstraints(
                            maxWidth:
                                (_controller.tabs.length * 150).toDouble())))
            : null,
        child: Align(
            alignment: Alignment.topCenter,
            child: Container(
              constraints:
                  BoxConstraints(maxWidth: ThemeService.containerWidth),
              child: _controller.tabs.length > 1
                  ? TabBarView(
                      children: _controller.tabs
                          .map<Widget>((tab) => _getTabData(tab))
                          .toList())
                  : _controller.tabs.length <= 0
                      ? SizedBox()
                      : _getTabData(_controller.tabs.first),
            )),
      ),
    );
  }

  TabBar _getTabBar() {
    return TabBar(
      automaticIndicatorColorAdjustment: true,
      indicatorSize: TabBarIndicatorSize.tab,
      isScrollable: _controller.tabs.length > 3 ? true : false,
      labelColor: ThemeService.mainColor,
      tabs: _controller.tabs
          .map((tab) => Tab(
                child: FittedBox(
                    fit: BoxFit.scaleDown,
                    child: Text(
                        LocalConfigService.isArabic ? tab.nameAr : tab.nameEn)),
              ))
          .toList(),
    );
  }

  Widget _getTabData(ViewTab tab) {
    if (_controller.properties.length == 0) return SizedBox();
    var props = _controller.getProps(tab);
    if (props.length == 0) {
      return SizedBox();
    }
    return SingleChildScrollView(child: _getData(props));
    // return SingleChildScrollView(
    //     child: props.length > 1
    //         ? Column(
    //             mainAxisAlignment: MainAxisAlignment.start,
    //             crossAxisAlignment: CrossAxisAlignment.start,
    //             children: group.map<Widget>((t) => _getData(t)).toList())
    //         : _getData(group.first));
  }

  _getData(List<Property> props) {
    props.sort((a, b) => a.editOrder - b.editOrder);
    return props.length == 1 // || isMobile
        ? LayoutBuilder(
            builder: (lbContext, boxConstraints) => Container(
                width: _getWidth(props.first.size, boxConstraints.maxWidth),
                margin: EdgeInsets.symmetric(horizontal: 10),
                child: PorpertyComponent(
                  readOnly: _controller.isReadOnly,
                  property: props.first,
                  isAddNew: _controller.isAddNew,
                  entity: _controller.entity,
                  //filterValue: _controller.data[props.first.filterValueField],
                  data: _controller.data!,
                  onChange: (v) => _controller.onValueChanging(
                      v,
                      props.first
                          .dbName), //_controller.data[props.first.dbName] = v,
                )))
        : LayoutBuilder(
            builder: (lbContext, boxConstraints) => Wrap(
              alignment: WrapAlignment.start,
              children: props
                  .map<Widget>((p) => Container(
                      width: _getWidth(p.size, boxConstraints.maxWidth),
                      margin: EdgeInsets.symmetric(horizontal: 10),
                      child: PorpertyComponent(
                        readOnly: _controller.isReadOnly,
                        isAddNew: _controller.isAddNew,
                        property: p,
                        entity: _controller.entity,
                        // onChange: (v) => _controller.data[p.dbName] = v,
                        //filterValue: _controller.data[p.filterValueField],
                        data: _controller.data!,
                        onChange: (v) =>
                            _controller.onValueChanging(v, p.dbName),
                      )))
                  .toList(),
            ),
          );
  }

  double? _getWidth(PropertySizes size, double fullwidth) {
    if (size == PropertySizes.MaxSize) return fullwidth;
    if (size == PropertySizes.xSmall) return 100;
    if (size == PropertySizes.Small) return 150;
    if (size == PropertySizes.Large) return 400;
    if (size == PropertySizes.xLarge) return 500;

    if (fullwidth < 400) {
      return fullwidth; // means take full width
    } else if (fullwidth < 850) {
      return (fullwidth / 2) - (_isMobile ? 20 : 20);
    }
    return (fullwidth / 3) - 20;
  }
}
